#ifndef _MS_H_
#define _MS_H_
#include "const.h"
#include <complex>
#include <vector>
#include <iostream>

#include <Eigen/Dense>

using namespace Eigen;


LL_CURVE_LOOK eesm_methology(INPUT_TO_LLS_MAPPING input, int _self_idx);
LL_CURVE_LOOK rbir_methology(INPUT_TO_LLS_MAPPING input, int _self_idx);
LL_CURVE_LOOK mib_methology(INPUT_TO_LLS_MAPPING input, int _self_idx);

class MS
{
    public:

		int self_idx;
		int self_sector_idx;
		int nearest_bs_idx;
		int supporting_bs_idx;
		LOCATION loc;

		void Configuration(int,int);
		void Set_MS_Location();
		void Channel_Update_MIMO(int);
		void Quantization_of_Ch(void);

		void CQI_Update(void);
		void Fourier_Transform_of_Channel(int);

		void Declare_ch_matrix(void);


		//void Precalculate_PMI(int, int);

		double speed;
		double moving_direction;
		double moving_direction_azimuth;

		//int num_compute_coef = 19;

		//mu mimo
		int                ** PMI;
		double             ** CQI;

		int    _self_idx;
		bool Indoor;

		//receive_downlink
		int self_ms_idx;
		void Receive_DL(int);
		void Find_Allocated_Rbs_Mcs(int);
		void Rate_matching(void);
		void Compute_Num_Bin_Symbol(void);
		void Compute_Code_Rate(void);
		void Compute_lls_mapping(void);
		void AddThroughput(double prob);
		double Return_initial_packet_error(void);

		void Kill_Memory(void);

		//void MMSE_Receiver(int, int);
		Eigen::MatrixXcd MMSE_Receiver (int, int);

		

		double compute_tone_SINR_MRC(int rb_idx);
		double compute_tone_SINR(int rb_idx);
		void compute_CQI_offset(double Received_SINR, int rb_idx);
		double Return_Throughput(int);

		std::vector <double> rbs_rx; 
		std::vector <int> rb_indices;

		double received_sinr_avg;

		double _prob_failure;
		//int num_re_tx;
		int total_num_tx;
		//bool ACK;
		//double     _throughput;
		int _info_bits;
		int failed;
		
		double _capacity;
		int _mcs_idx;
		int _cqi_idx;
		int _mod_type;
		int num_rx_rb;
		int _num_traffic;
		double _code_rate;
		double _snr_scale_factor;
		//double UE_ICI;
		//int num_added;
		//double CQI_offset;

		SCHEDULE_DECISION  ** ppSchedulerRead;
		//double UE_PATH_LOSS;


		LL_CURVE_LOOK _input2table;// modulation_idx , effective_code_rate, snr 
		INPUT_TO_LLS_MAPPING Return_mapping_info(void);
		
		// cqi_offset
		double CQI_offset;
		int num_added;

		//Declare
		//complex <double> *** H_m;
		//complex <double> **** HI_m;

		//Declare Channel Matrix
		MatrixXcd *H_m;
		MatrixXcd **HI_m;


		double BLER_SUM_Value;
		int BLER_SUM_count;

		double decision_BLER_SUM_Value;
		int decision_BLER_SUM_count;

		void Allocate_memory(void);
		void Delete_memory(void);
		//int *propagation_condition;
		int floor_idx;
		double MS_HEIGHT_FINAL;
		LOCATION pos_indoor;
		LOCATION *pos;
		ArrayXd  *TLSPs;
		ArrayXd  *LSPs;

		//double *sigma_SF;
		//double *shadow_fading;


};

#endif

double ComputeBLER(LL_CURVE_LOOK);
void Get_Geometry(double geometry_ue, int ue_idx);
void Coupling_Loss_Get(double cp_ue, int ue_idx);
void Get_RMS_delay_spread(double RMS_delay_spread_ue, int ue_idx);
void Get_circular_angle_spread_AOA(double angle_spread_AOA_ue, int ue_idx);
void Get_circular_angle_spread_AOD(double angle_spread_AOD_ue, int ue_idx);